/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdampluckedstring.h"

static int  gdam_plucked_string_append_start_request (GdamFilter* plucked,
						      int         sub_id,
						      GdamRequestBuilder*);

static void gdam_plucked_string_init(GdamPluckedString* plucked_string)
{
	plucked_string->hertz = 440.0;
	plucked_string->gain = 0.75;
	plucked_string->filter_coeff = 1.0;
}

static void gdam_plucked_string_arg_set_hertz(GtkObject* object, GdamArg* arg)
{
	gdam_plucked_string_set_hertz( GDAM_PLUCKED_STRING(object),
	                               GDAM_ARG_DOUBLE(arg) );
}
static void gdam_plucked_string_arg_get_hertz(GtkObject* object, GdamArg* arg)
{
	GDAM_ARG_DOUBLE(arg) = GDAM_PLUCKED_STRING(object)->hertz;
}
static void gdam_plucked_string_arg_set_gain(GtkObject* object, GdamArg* arg)
{
	gdam_plucked_string_set_gain( GDAM_PLUCKED_STRING(object),
	                              GDAM_ARG_DOUBLE(arg) );
}
static void gdam_plucked_string_arg_get_gain(GtkObject* object, GdamArg* arg)
{
	GDAM_ARG_DOUBLE(arg) = GDAM_PLUCKED_STRING(object)->gain;
}
static void gdam_plucked_string_arg_set_filter_coeff(GtkObject* object, GdamArg* arg)
{
	gdam_plucked_string_set_filter_coeff( GDAM_PLUCKED_STRING(object),
	                                      GDAM_ARG_DOUBLE(arg) );
}
static void gdam_plucked_string_arg_get_filter_coeff(GtkObject* object, GdamArg* arg)
{
	GDAM_ARG_DOUBLE(arg) = GDAM_PLUCKED_STRING(object)->filter_coeff;
}


static void gdam_plucked_string_class_init(GdamPluckedStringClass* klass)
{
	GtkObjectClass* object_class = GTK_OBJECT_CLASS(klass);
	GDAM_FILTER_CLASS(klass)->append_start_request
				= gdam_plucked_string_append_start_request;
	gdam_arg_register(object_class,
	                  "hertz",
			  GDAM_ARG_TYPE_DOUBLE,
			  gdam_plucked_string_arg_set_hertz,
			  gdam_plucked_string_arg_get_hertz,
			  GDAM_ARG_VITAL);
	gdam_arg_register(object_class,
	                  "gain",
			  GDAM_ARG_TYPE_DOUBLE,
			  gdam_plucked_string_arg_set_gain,
			  gdam_plucked_string_arg_get_gain,
			  GDAM_ARG_VITAL);
	gdam_arg_register(object_class,
	                  "filter_coeff",
			  GDAM_ARG_TYPE_DOUBLE,
			  gdam_plucked_string_arg_set_filter_coeff,
			  gdam_plucked_string_arg_get_filter_coeff,
			  GDAM_ARG_VITAL);
}

GtkType gdam_plucked_string_get_type()
{
	static GtkType plucked_string_type = 0;
	if (!plucked_string_type) {
		static const GtkTypeInfo plucked_string_info = {
			"GdamPluckedString",
			sizeof(GdamPluckedString),
			sizeof(GdamPluckedStringClass),
			(GtkClassInitFunc) gdam_plucked_string_class_init,
			(GtkObjectInitFunc) gdam_plucked_string_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_FILTER;
		plucked_string_type = gtk_type_unique(parent,
		                                 &plucked_string_info);
	}
	return plucked_string_type;
}

static void plucked_string_successful_creation(int id, gpointer ptr)
{
	gdam_model_set_source_id(GDAM_MODEL(ptr), id);
	gtk_object_unref(GTK_OBJECT(ptr));
}
static int  gdam_plucked_string_append_start_request (GdamFilter* this_filter,
						      int         sub_id,
						    GdamRequestBuilder* builder)
{
	GdamPluckedString* plucked = GDAM_PLUCKED_STRING(this_filter);
	gtk_object_ref(GTK_OBJECT(this_filter));
	return gdam_request_builder_append_create(
		builder,
		gdam_request_create_source("GdamPluckedString",
					  plucked_string_successful_creation,
					  (GHookFunc) gtk_object_unref,
					  this_filter,
	                                  4,
	                                  "gain",
					  GDAM_ARG_TYPE_DOUBLE,
					  plucked->gain,
	                                  "hertz",
					  GDAM_ARG_TYPE_DOUBLE,
					  plucked->hertz,
	                                  "filtered",
					  GDAM_ARG_TYPE_INT,
					  sub_id,
					  "filter_coeff",
					  GDAM_ARG_TYPE_DOUBLE,
					  plucked->filter_coeff)
	);
}

static void do_nothing()
{
}

void gdam_plucked_string_set_gain (GdamPluckedString* string,
                                   gdouble            gain)
{
	int source_id = GDAM_MODEL(string)->source_id;
	if (source_id == 0)
		return;
	string->gain = gain;
	gdam_channel_append_request(
		GDAM_MODEL(string)->channel,
		gdam_request_set_args(
			source_id,
			1,
			gdam_arg_new_from_valist(1, 
			                         "gain",
			                         GDAM_ARG_TYPE_DOUBLE,
						 string->gain),
			do_nothing,
			do_nothing,
			NULL
		)
	);
}

void gdam_plucked_string_set_hertz(GdamPluckedString* string,
                                   gdouble            hertz)
{
	int source_id = GDAM_MODEL(string)->source_id;
	if (source_id == 0)
		return;
	string->hertz = hertz;
	gdam_channel_append_request(
		GDAM_MODEL(string)->channel,
		gdam_request_set_args(
			source_id,
			1,
			gdam_arg_new_from_valist(1, 
			                         "hertz",
			                         GDAM_ARG_TYPE_DOUBLE,
						 string->hertz),
			do_nothing,
			do_nothing,
			NULL
		)
	);
}
void gdam_plucked_string_set_filter_coeff(GdamPluckedString* string,
                                          gdouble            filter_coeff)
{
	int source_id = GDAM_MODEL(string)->source_id;
	if (source_id == 0)
		return;
	string->filter_coeff = filter_coeff;
	gdam_channel_append_request(
		GDAM_MODEL(string)->channel,
		gdam_request_set_args(
			source_id,
			1,
			gdam_arg_new_from_valist(1, 
			                         "filter_coeff",
			                         GDAM_ARG_TYPE_DOUBLE,
						 string->filter_coeff),
			do_nothing,
			do_nothing,
			NULL
		)
	);
}

void gdam_plucked_string_set_pluck(GdamPluckedString* string,
                                   gdouble            pluck_power)
{
	string->pluck_power = pluck_power;
}

void gdam_plucked_string_pluck    (GdamPluckedString* string)
{
	int source_id = GDAM_MODEL(string)->source_id;
	if (source_id == 0)
		return;
	gdam_channel_append_request(
		GDAM_MODEL(string)->channel,
		gdam_request_set_args(
			source_id,
			1,
			gdam_arg_new_from_valist(1, 
			                         "pluck",
			                         GDAM_ARG_TYPE_DOUBLE,
						 string->pluck_power),
			do_nothing,
			do_nothing,
			NULL
		)
	);
}
