
#include "gdamdeviceswitcherskin.h"
#include "gdampacker.h"
#include "gdamskinutil.h"
#include <string.h>
#include <stdlib.h>
#include <gtk/gtksignal.h>
#include <gtk/gtktogglebutton.h>

static void gdam_device_switcher_skin_init(GdamDeviceSwitcherSkin* device_switcher_skin)
{
}
static void gdam_device_switcher_skin_class_init(GdamDeviceSwitcherSkinClass* klass);

GtkType gdam_device_switcher_skin_get_type()
{
	static GtkType device_switcher_skin_type = 0;
	if (!device_switcher_skin_type) {
		static const GtkTypeInfo device_switcher_skin_info = {
			"GdamDeviceSwitcherSkin",
			sizeof(GdamDeviceSwitcherSkin),
			sizeof(GdamDeviceSwitcherSkinClass),
			(GtkClassInitFunc) gdam_device_switcher_skin_class_init,
			(GtkObjectInitFunc) gdam_device_switcher_skin_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_SKIN;
		device_switcher_skin_type = gtk_type_unique(parent,
		                                 &device_switcher_skin_info);
	}
	return device_switcher_skin_type;
}

typedef struct _SwitchInfo{
	char*			device_name;
	GdamDeviceSwitcherSkin* skin;
} SwitchInfo;

static void on_gdam_device_switcher_skin_select_device(GtkObject* object,
						       SwitchInfo* info)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object))){
		GdamToplevel* toplevel;
		GdamModel* model;
		GdamSkin* skin;
		skin = GDAM_SKIN(info->skin);
		toplevel = gdam_toplevel_get_global(skin->channel);
		g_return_if_fail (info->skin->child_skin != 0);
			
		model = GDAM_SKIN(info->skin->child_skin)->model;
		gdam_debug("SWITCHER set device: %s", info->device_name);
		gdam_toplevel_set_child_device( toplevel, 
						model, 
						info->device_name);
	}
}
	

typedef struct _SetChildInfo SetChildInfo;
struct _SetChildInfo {
	char* switcher_name;
	char* child_name;
	int index;
	GdamPacker* packer;
};

static void gdam_device_switcher_skin_set_child(
				GdamDeviceSwitcherSkin* skin,
				guint index,
				GdamSkin* child_skin)
{
	if (index > 0){
		g_warning("device switcher set child: %d out of range", index);
		return;
	}
	gdam_debug("Setting child skin");

	skin->child_skin = child_skin;
}

static gboolean deferred_set_child(GdamSkin* skin, gpointer ptr)
{
	SetChildInfo* info = ptr;
	GdamDeviceSwitcherSkin* switcher;
	GdamSkin* child_skin;
	GdamPacker* packer = GDAM_PACKER(skin);

	switcher = GDAM_DEVICE_SWITCHER_SKIN(gdam_packer_get_skin(packer,
							info->switcher_name));
	child_skin = gdam_packer_get_skin(packer, info->child_name);
	if (switcher == NULL) {
		g_warning("set_child: switcher `%s' not found",
		                                info->switcher_name);
		return FALSE;
	}

	gdam_device_switcher_skin_set_child(switcher, 
					    info->index, 
					    child_skin);
	return TRUE;
}

static void free_set_child(gpointer ptr)
{
	SetChildInfo* info = ptr;
	g_free(info->switcher_name);
	g_free(info->child_name);
	g_free(info);
}

static gboolean gdam_device_switcher_select_device_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
	SwitchInfo* info;
	if (num_args <2) {
		g_warning("%s: too few arguments", args[0]);
		return TRUE;
	}
	info = g_new(SwitchInfo,1);
	info->skin = GDAM_DEVICE_SWITCHER_SKIN(skin);
	info->device_name = strdup(args[1]);
	gtk_signal_connect(object,
			   signal_name,
			   on_gdam_device_switcher_skin_select_device,
			   info);
	return TRUE;
}
static void toggle_misses_data(GtkToggleButton* toggle, 
                               GdamSkin* skin)
{
	GdamDeviceSwitcherSkin* switcher_skin;
	GdamModel* component;
	GdamToplevel* toplevel;
	switcher_skin = GDAM_DEVICE_SWITCHER_SKIN(skin);
	component = GDAM_SKIN(switcher_skin->child_skin)->model;
	toplevel = gdam_toplevel_get_global(skin->channel);
	g_return_if_fail(toplevel != NULL);
	gdam_toplevel_set_misses_data(toplevel, component, toggle->active);
}

static gboolean gdam_device_switcher_toggle_misses_data_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
	g_return_val_if_fail(GTK_IS_TOGGLE_BUTTON(object), TRUE);
	gtk_signal_connect(object,
			   signal_name,
			   GTK_SIGNAL_FUNC(toggle_misses_data),
			   skin);
	return TRUE;
}

static gboolean gdam_packer_device_switcher_set_child_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
	SetChildInfo* info;
	if (num_args <4) {
		g_warning("%s: too few arguments", args[0]);
		return TRUE;
	}
	info = g_new(SetChildInfo, 1);
	info->child_name = args[3];
	info->index = atoi(args[2]);
	info->switcher_name = args[1];
	args[1] = args[3] = NULL;
	
	gdam_skin_add_initializer(skin,
	                          deferred_set_child,
		                  free_set_child,
			          info);
	return TRUE;
}

static GdamSkinGenericConnect connect_infos[] =
{ 	{ "gdam_device_switcher_select_device",
	   gdam_device_switcher_select_device_handler },
	{ "gdam_device_switcher_toggle_misses_data",
	   gdam_device_switcher_toggle_misses_data_handler }
};

static GdamSkinGenericConnect packer_connect_infos[] =
{
	{ "gdam_packer_device_switcher_set_child",
	   gdam_packer_device_switcher_set_child_handler },
};


static void gdam_device_switcher_skin_class_init(GdamDeviceSwitcherSkinClass* klass)
{
	GdamSkinClass* skin_class = GDAM_SKIN_CLASS(klass);
	//skin_class->build_new_model = gdam_gain_build_new_model;
	gdam_skin_class_register_generic_handlers(skin_class,
				                  G_N_ELEMENTS(connect_infos),
				                  connect_infos);
	gdam_skin_class_register_generic_handlers(
					gdam_packer_class(),
				        G_N_ELEMENTS(packer_connect_infos),
				        packer_connect_infos);
}
